/*
TI83 Plus File Squisher version 0.2b by Tom Lutz (tlutz@stevens-tech.edu)
I don't mind if you use my code or classes but if you use them I would appreciate an e-mail
letting me know what you're doing (solely for the purposes of curiosity and feedback).
Also, if you wind up using them, please give me at least a little credit! :-)

This program may contain some bugs and errors. If you find any, please e-mail me.
I will promptly fix it and re-release a new version.

The command line parameters for this are as follows:
-<file type> <source file> <target file>
Source file type is either "ZDS" or "8XP". I will consider adding .83P file type when
I get a little more time on my hands. If someone would like to expedite this, e-mail
me a couple of .83P files so I can check them out.

If the file type is "ZDS", the source file must be a Zilog Developer Studio hex file, or a
compatible type.

If the file type is "8XP", the source file must be an unsquished 83 plus program file.



***************Notes about the classes***************

--------------
TI83PFile
--------------
This class takes several inputs:
	-Archived Flag
	-Variable Data
	-Variable Data Type
	-Variable Name
	-Version
	-File Comment
After a new instance of the TI83PFile class is started, you can get a string
representation of the actual .8XP file. It calculates the checksum and formats
the data to TI specifications. See info on the StrWriter class to see what
this actually is used for.

--------------
StrWriter
--------------
This class takes a String (must be an even number of characters, 0-9 and A-F)
that is a hex representation of a file. You can convert the string to a byte array
or write the byte array directly to a file.

*/


import java.io.*;
import TI83PFile;
import StrWriter;

public class SquishIt{

	public static void main(String args[]) throws IOException{
		if(args.length != 3){
			System.out.println(new String("Invalid Input\n\nUsage is -<file input type> <source> <destination>\n"+
			"\nFile Input Types:\n\t-ZDS\tUse a HEX file outputted by Zilog Developer Studio\n"+
			"\t-8XP\tUse an unsquished TI-GraphLink program file\n"+
			"Send comments, suggestions, harsh criticisms and profanities to:\n"+
			"tlutz@stevens-tech.edu\n\n\n"+
			"Built on 1/23/2002"));
			return;
		}

	String ProgramDataString = new String("BB6D");
	String Temp = new String("GARBAGEGARBAGE");
	FileReader SourceFile;

		if(args[0].equalsIgnoreCase("-zds"))
		{
			System.out.println("Processing HEX file: "+args[1]);
			try
			{
			SourceFile = new FileReader(args[1]);
			}
			catch(Exception FileNotFoundException)
			{
				System.out.println("Source file not found.");
				return;
			}

			while(Temp.length() > 11)
			{
				Temp = readLine(SourceFile);
				//System.out.println(Temp);
				if(Temp.length() > 11)
				ProgramDataString += Temp.substring(9,Temp.length()-2);
			}

			//System.out.println("Length of Data: "+ProgramDataString.length());
			//System.out.println(ProgramDataString.substring(0,80));


			TI83PFile Prog = new TI83PFile(6,ProgramDataString,GetProgName(args[1]),"Generated by SquishIt beta 0.2",1,false);
			StrWriter SW = new StrWriter(Prog.getFileDataChar(),args[2]);
			SW.WriteToFile();
		}
		else if(args[0].equalsIgnoreCase("-8xp"))
		{
			System.out.println("Processing 8xp file: "+args[1]);
			try
			{
			SourceFile = new FileReader(args[1]);
			}
			catch(Exception FileNotFoundException)
			{
				System.out.println("Source file not found.");
				return;
			}

			SourceFile.skip((long)59);
			if(SourceFile.read() != 5)
			{
				System.out.println("\nInvalid File Format (Data type doesn't match)");
				return;
			}

			SourceFile.skip((long)12);
			int size = (int)SourceFile.read();
			size += (int)SourceFile.read()*256;

			byte[] temp = new byte[2];

			temp[0] = (byte)SourceFile.read();
			temp[1] = (byte)SourceFile.read();

			if(!(temp[0] == -69 && temp[1] == 108)){
				System.out.println("\nSource file is not an assembly program.");
				return;
			}
			size = size - 4;

			ProgramDataString = new String("BB6D");
			char achar;

			while(size > 0){
				achar = (char)SourceFile.read();
				if(((achar >= 'A') && (achar <= 'Z')) || ((achar >= '0') && (achar <= '9')))
					ProgramDataString += achar;
				size--;
			}

			TI83PFile Prog = new TI83PFile(6,ProgramDataString,GetProgName(args[1]),"Generated by SquishIt beta 0.2",1,false);
			StrWriter SW = new StrWriter(Prog.getFileDataChar(),args[2]);
			SW.WriteToFile();





			return;
		}
		return;
	}

	public static String readLine(FileReader Source) throws IOException{
		String temp = new String();
		char tempchar = (char)Source.read();

		while(tempchar != 0x0D)
		{
			temp += tempchar;
			tempchar = (char)Source.read();
		}

		tempchar = (char)Source.read();

		return temp;
	}

	private static String GetProgName(String Filename){
		int indexstart = Filename.length()-1;
		int indexend;
		char a = 'a';

		while((a != ':') && (a != '\\') && (indexstart >= 0)){
			a = Filename.charAt(indexstart);
			--indexstart;
		}

		indexstart++;
		if(Filename.charAt(indexstart) == '\\')
			indexstart++;
		indexend = indexstart+1;

		while((Filename.charAt(indexend) != '.') && (indexend < Filename.length())){
			indexend++;
		}
		if(indexend == Filename.length())
			indexend--;

		return Filename.substring(indexstart,indexend);
	}
}

