public class TI83PFile
{
	private String ObjectData;
	private int ObjectType;
	private String ObjectName;
	private String CObjectName;
	private String Comment = new String("");
	private String ConvertedComment = new String("");
	private boolean Archived = false;
 	private int Version = 0;
 	private final String Header = new String("2A2A54493833462A1A0A00"); //"**TI83F*..."
 	private int MsgBlock2Length;
 	private int ObjectSize;
 	private int ODBCheckSum;
 	private int FileDataCount;
 	private String FileData;


	public TI83PFile(int ObjectType, String ObjectData, String ObjectName)
	{
		this.ObjectData = ObjectData;
		this.ObjectType = ObjectType;
		this.ObjectName = ObjectName;
	}

	public TI83PFile(int ObjectType, String ObjectData, String ObjectName, String Comment)
	{
		this.ObjectData = ObjectData;
		this.ObjectType = ObjectType;
		this.ObjectName = ObjectName;
		this.Comment = Comment;
		Convert();
	}

	public TI83PFile(int ObjectType, String ObjectData, String ObjectName, String Comment, int Version, boolean Archived)
	{
		this.ObjectData = ObjectData;
		this.ObjectType = ObjectType;
		this.ObjectName = ObjectName;
		this.Comment = Comment;
		this.Version = Version;
		this.Archived = Archived;
		Convert();
	}


	public void SetArchived(boolean Archived)
	{
		this.Archived = Archived;
	}

	public void SetComment(String Comment)
	{
		this.Comment = Comment;
		Convert();
	}

	public void SetVersion(int Version)
	{
		this.Version = Version;
	}

	public String getName()
	{
		return ObjectName;
	}

	public String getData()
	{
		return ObjectData;
	}

	public String getFileDataChar()
	{

		FileData = new String(Header);

		//System.out.println("Header:" + Header);

		FileData += ConvertedComment;

		//System.out.println("Comment:" + ConvertedComment);

		//System.out.println("LSB: " + LSB(ObjectData.length()/2+19));
		//System.out.println("MSB: " + MSB(ObjectData.length()/2+19));

		//System.out.println("Object name: " + CObjectName);
		//System.out.println("LSB OF VERSION: " + LSB(Version));
		//System.out.println("LSB OF OBJECT TYPE: " + LSB(ObjectType));


		FileData += LSB(ObjectData.length()/2+19);
		FileData += MSB(ObjectData.length()/2+19);
		FileData += "0D00";

		FileData += LSB(ObjectData.length()/2+2); //length of data block + object size bytes
		FileData += MSB(ObjectData.length()/2+2);
		FileData += LSB(ObjectType);
		FileData += CObjectName;
		FileData += LSB(Version);

		if(Archived)
			FileData += "01";
		else
			FileData += "00";

		FileData += LSB(ObjectData.length()/2+2);	//Msg block 2 length
		FileData += MSB(ObjectData.length()/2+2);
		FileData += LSB(ObjectData.length()/2);		//Object size
		FileData += MSB(ObjectData.length()/2);

		FileData += ObjectData;

		FileData += CheckSum();

		return FileData;
	}

	//Convert Comment into an 84 char string
	//Convert ObjectName into a 16 char string
	private void Convert()
	{
		String temp = new String(), temp2 = new String();

		for(int i=0; i<42; i++)
		{
			if(i >= Comment.length())
				temp += "00";
			else
				temp += CharToHex(Comment.charAt(i));
		}

		ObjectName = ObjectName.toUpperCase();

		for(int i=0; i<8; i++)
		{
			if(i >= ObjectName.length())
				temp2 += "00";
			else
				temp2 += CharToHex(ObjectName.charAt(i));
		}
		ConvertedComment = temp;
		CObjectName = temp2;
	}

	//Given a character, returns a two byte string that represents its hex byte value
	private String CharToHex(char c)
	{
		String result = new String();
		result += DigToChar((int)c/16);
		result += DigToChar((int)c%16);
		return result;
	}

	//returns a character representing the hex value of an int from 0 to 15
	private char DigToChar(int n)
	{
		while(n > 15)
		{
			n -= 10;
		}

		char[] digarray = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
		return digarray[n];
	}

	private int CharToDig(char c)
	{
		if(c == 'A')
			return 10;
		else if(c == 'B')
			return 11;
		else if(c == 'C')
			return 12;
		else if(c == 'D')
			return 13;
		else if(c == 'E')
			return 14;
		else if(c == 'F')
			return 15;
		else return (int)c-(int)'0';
	}

	private String LSB(int n)
	{
		String result = new String();
		result += DigToChar(n/4096);
		n = n%4096;
		result += DigToChar(n/256);
		n = n%256;
		result += DigToChar(n/16);
		n = n%16;
		result += DigToChar(n);
		String finalresult = new String();
		finalresult += result.charAt(2);
		finalresult += result.charAt(3);
		return finalresult;
	}

	private String MSB(int n)
	{
		String result = new String();
		result += DigToChar(n/4096);
		n = n%4096;
		result += DigToChar(n/256);
		n = n%256;
		result += DigToChar(n/16);
		n = n%16;
		result += DigToChar(n);
		String temp = new String();
		temp += result.charAt(0);
		temp += result.charAt(1);
		return temp;
	}

	//Add up hex values in filedata from index 54 to the end...convert that int to a
	//2 byte hex string
	private String CheckSum()
	{
		String temp = new String();
		int res = 0;
		for(int a=110; a<FileData.length()-1; a = a+2)
		{
			temp = new String();
			temp += FileData.charAt(a);
			temp += FileData.charAt(a+1);
			//System.out.println("temp string for checksum: " + temp);
			res += Str2Dec(temp);
			//System.out.println("result integer for checksum: " + res);
		}
//
		res = res%65536;

		//System.out.println("Final checksum int value: " + res);
//
		String temp2 = new String(LSB(res));
		temp2 += MSB(res);
		//System.out.println("Final checksum hex value: " + temp2);
		return temp2;
	}




	//Accepts 2 byte string and converts that hex number into decimal
	private int Str2Dec(String str)
	{

		int answer = 0;

		if(str.charAt(0) == 'A')
			answer += 10*16;
		else if(str.charAt(0) == 'B')
			answer += 11*16;
		else if(str.charAt(0) == 'C')
			answer += 12*16;
		else if(str.charAt(0) == 'D')
			answer += 13*16;
		else if(str.charAt(0) == 'E')
			answer += 14*16;
		else if(str.charAt(0) == 'F')
			answer += 15*16;
		else
			answer += (((int)(str.charAt(0))-(int)'0')) * 16;

		if(str.charAt(1) == 'A')
			answer += 10;
		else if(str.charAt(1) == 'B')
			answer += 11;
		else if(str.charAt(1) == 'C')
			answer += 12;
		else if(str.charAt(1) == 'D')
			answer += 13;
		else if(str.charAt(1) == 'E')
			answer += 14;
		else if(str.charAt(1) == 'F')
			answer += 15;
		else
			answer += ((int)(str.charAt(1))-(int)'0');

		//System.out.println("Hex data: " + str + "   Decimal: " + answer);

		return answer;
	}

}




